// PlaylistDlg.cpp : implementation file
//

#include "stdafx.h"
#include "SysTrayPlay.h"
#include "PlaylistDlg.h"
#include "MainWnd.h"
#include "TagEditor.h"
#include "SeekDialog.h"
#include "MP3InfoDlg.h"
#include "RenameDlg.h"
#include "MainHelper.h"
#include "PlaylistManager.h"
#include "In/MultiDllInput.h"
#include "Out/MultiDllOutput.h"
#include "WAEmuWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CSysTrayPlayApp theApp;

extern CMainWnd* pMainFrame;
extern CPlaylistDlg* PlaylistDlg;
extern CSeekDialog* SeekDialog;
extern CMP3InfoDlg* MP3InfoDlg;
extern CWAEmuWnd* WAEmuWnd;

extern Helper m_Helper;
extern PlaylistManager m_PlaylistManager;
extern CMultiDllInput* stdIn;
extern CMultiDllOutput* stdOut;

static int __stdcall BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	CString dir;
	dir = pMainFrame->InitDir;
	if (dir.Right(1) == "\\")
		dir = dir.Left(dir.GetLength() - 1);

	if (uMsg == BFFM_INITIALIZED)
		::SendMessage(hwnd, BFFM_SETSELECTION, TRUE, (LPARAM)LPCTSTR(dir));
	return 0;
}

void CSrchEdit::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	switch (nChar)
	{
	case VK_UP:
	case VK_DOWN:
	case VK_PRIOR:
	case VK_NEXT:
	case VK_HOME:
	case VK_END:
		PlaylistDlg->GetDlgItem(IDC_PLAYLIST_AT)->SetFocus();
		PlaylistDlg->GetDlgItem(IDC_PLAYLIST_AT)->PostMessage(WM_KEYDOWN, nChar, nRepCnt + 0x10000 * nFlags);
		return;
		break;
	}
	CEdit::OnKeyDown(nChar, nRepCnt, nFlags);
}

/////////////////////////////////////////////////////////////////////////////
// CPlaylistDlg dialog


CPlaylistDlg::CPlaylistDlg(UINT id, CWnd* pParent /*=NULL*/) : CDialog(id, pParent)
{
	menu = new CMenu();
	CanResize = false;
	RemoveneThreadHan = NULL;
	RemoveDuplicateEntriesThreadHan = NULL;
	SortThreadHan = NULL;
}

CPlaylistDlg::~CPlaylistDlg()
{
	menu->DestroyMenu();
	delete menu;
	PlaylistDlg = NULL;
}

BEGIN_MESSAGE_MAP(CPlaylistDlg, CDialog)
	ON_WM_CLOSE()
	ON_COMMAND(ID_FILE_ADDFILE, OnFileAddFile)
	ON_COMMAND(ID_FILE_ADDFOLDER, OnFileAddFolder)
	ON_COMMAND(ID_FILE_EDITTAG, OnFileEditTag)
	ON_COMMAND(ID_FILE_REMOVE, OnFileRemove)
	ON_COMMAND(ID_PLAYLIST_OPEN, OnPlaylistOpen)
	ON_COMMAND(ID_PLAYLIST_SAVE, OnPlaylistSave)
	ON_COMMAND(ID_ARRANGE_SHUFFLE, OnArrangeShuffle)
	ON_COMMAND(ID_ARRANGE_SORT, OnArrangeSort)
	ON_COMMAND(ID_ARRANGE_SORTBYFILENAME, OnArrangeSortByFilename)
	ON_COMMAND(ID_TRACKS_SELECTION_ADDTOEXISTENTPLAYLIST, OnTracksSelectionAddToExistentPlaylist)
	ON_COMMAND(ID_TRACKS_SELECTION_CREATENEWPLAYLIST, OnTracksSelectionCreateNewPlaylist)
	ON_COMMAND(ID_TRACKS_ADDPLAYLIST, OnTracksAddPlaylist)
	ON_LBN_KILLFOCUS(IDC_PLAYLIST_AT, OnKillFocusPlaylistAt)
	ON_COMMAND(ID_ARRANGE_REMOVENE, OnArrangeRemovene)
	ON_COMMAND(ID_ARRANGE_REMOVEDUPLICATEENTRIES, OnArrangeRemoveDuplicateEntries)
	ON_EN_CHANGE(IDC_SEARCH, OnChangeSearch)
	ON_COMMAND(ID_TRACKS_REMOVEALL, OnTracksRemoveAll)
	ON_WM_SIZE()
	ON_WM_SHOWWINDOW()
	ON_COMMAND(ID_POPUP_SELECTALL, OnPopupSelectAll)
	ON_COMMAND(ID_TRACKS_REREADALLTAGS, OnTracksReReadAllTags)
	ON_COMMAND(ID_TRACKS_RENAMEFILES, OnTracksRenameFiles)
	ON_COMMAND(ID_TRACKS_FILEMANAGEMENT_DELETE, OnTracksFileManagementDelete)
	ON_COMMAND(ID_TRACKS_FILEMANAGEMENT_COPYTO, OnTracksFileManagementCopyTo)
	ON_COMMAND(ID_TRACKS_FILEMANAGEMENT_MOVETO, OnTracksFileManagementMoveTo)
	ON_COMMAND(ID_FILE_SAVEINEXTENDEDFORMAT, OnFileSaveInExtendedFormat)
	ON_WM_ENTERMENULOOP()
	ON_WM_DROPFILES()
	ON_WM_SYSCOMMAND()
	ON_COMMAND_RANGE(IDM_TRACKS,IDM_TRACKS+49999,OnTrackMenu)
	ON_MENUXP_MESSAGES()
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP(CPLLB,CListBox)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_PAINT()
	ON_WM_ERASEBKGND( )
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP(CSrchEdit,CEdit)
	ON_WM_KEYDOWN()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPlaylistDlg message handlers

void CPlaylistDlg::DoDataExchange(CDataExchange* pDX)
{
	CMenuXP::SetXPLookNFeel(this, pMainFrame->XPLookNFeel);
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_FILENAME, m_filename);
	DDX_Control(pDX, IDC_PLAYLIST_AT, m_playlistat);
	DDX_Control(pDX, IDC_SEARCH, m_search);
}


CPlaylistDlg* CPlaylistDlg::getInstance(UINT id)
{
	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		PlaylistDlg->SaveWindowPosAndDestroy();
	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd == NULL)
	{
		delete PlaylistDlg;
		PlaylistDlg = NULL;
	}
	if (PlaylistDlg == NULL)
	{
		PlaylistDlg = new CPlaylistDlg(id);
		PlaylistDlg->Create(id);
	}
	else
		PlaylistDlg->SetFocus();
	return PlaylistDlg;
}

bool CPlaylistDlg::FileExists(CString fname)
{
	if (fname.Left(7).CompareNoCase(_T("http://")) == 0 || fname.Left(6).CompareNoCase(_T("ftp://")) == 0)
		return true;
	CFileFind finder;
	if (finder.FindFile(fname))
	{
		finder.FindNextFile();
		return(finder.GetLength() > 0);
	}
	else
		return false;
}

void CPlaylistDlg::FillPlaylist()
{
	m_playlistat.ResetContent();
	for (UINT i = 0; i < (UINT)m_PlaylistManager.GetSize(); i++)
		m_playlistat.AddString(_T(""));
	ShowFileName();
	m_playlistat.Invalidate();
}

void CPlaylistDlg::FindMP3Files(CString path)
{
	CFileFind finder;
	CString fp, fext;
	CString posExt = stdIn->GetFileExtensions();
	BOOL bWorking = finder.FindFile(path + "*.*");
	while (bWorking)
	{
		bWorking = finder.FindNextFile();
		fp = finder.GetFilePath();
		fext = fp.Mid(fp.ReverseFind('.') + 1, 3);fext.MakeUpper();
		if (posExt.Find(fext) != -1)
		{
			if (finder.GetLength() > 0)
			{
				m_PlaylistManager.Add(fp);
				m_playlistat.AddString(_T(""));
				continue;
			}
		}
		if (finder.IsDirectory() && !finder.IsDots())
			FindMP3Files(fp + "\\");
	}
}

void CPlaylistDlg::OnArrangeShuffle()
{
	if (m_playlistat.GetCount() < 2)
		return;

	UINT i, j, k, sl;
	for (i = 0; i < (UINT)m_PlaylistManager.GetSize() * 1.5 + 1; i++)
	{
		j = rand() % m_PlaylistManager.GetSize();
		k = rand() % m_PlaylistManager.GetSize();
		CachedTag tmp = m_PlaylistManager.Get(k);
		m_PlaylistManager.Add(m_PlaylistManager.Get(j), k);
		m_PlaylistManager.Add(tmp, j);
		sl = m_playlistat.GetSel(j); m_playlistat.SetSel(j, m_playlistat.GetSel(k)); m_playlistat.SetSel(k, sl);
		if ((UINT)m_playlistat.GetCaretIndex() == j)
			m_playlistat.SetCaretIndex(k, TRUE);
		else if ((UINT)m_playlistat.GetCaretIndex() == k)
			m_playlistat.SetCaretIndex(j, TRUE);
		if ((UINT)pMainFrame->Current == j)
			pMainFrame->Current = k;
		else if (pMainFrame->Current == k)
			pMainFrame->Current = j;
	}

	ShowFileName();
	m_playlistat.SetTopIndex(m_playlistat.GetFirstSelected() != -1 ? m_playlistat.GetFirstSelected() : pMainFrame->Current);
	m_playlistat.Invalidate();
}

unsigned long __stdcall RemoveneThread()
{
	int i = 0;
	bool CurDeleted = false;
	//Dsactivation du rafraichissement
	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		PlaylistDlg->m_playlistat.SetRedraw(FALSE);
	while (i < m_PlaylistManager.GetSize() && m_PlaylistManager.GetSize() > 0)
	{
		if (!PlaylistDlg->FileExists(m_PlaylistManager.GetPlaylist(i)))
		{
			m_PlaylistManager.RemoveAt(i);
			if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
				PlaylistDlg->m_playlistat.DeleteString(i);
			if ((UINT)i < pMainFrame->Current)
				pMainFrame->Current--;
			else if ((UINT)i == pMainFrame->Current && !CurDeleted)
				CurDeleted = true;
		}
		else
			i++;
	}
	if (CurDeleted)
	{
		pMainFrame->Current = 0;
		if (m_PlaylistManager.GetSize() > 0)
		{
			if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
				::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
		}
		else
		{
			if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
				::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
		}
		pMainFrame->UpdMBTR();
	}

	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
	{
		PlaylistDlg->m_playlistat.SetSel(-1, FALSE);
		PlaylistDlg->ShowFileName();
		//Activation du rafraichissement
		PlaylistDlg->m_playlistat.SetRedraw(TRUE);
		PlaylistDlg->m_playlistat.Invalidate();
	}

	PlaylistDlg->RemoveneThreadHan = NULL;
	return TRUE;
}

bool CPlaylistDlg::StopRemovene()
{
	bool toret = (RemoveneThreadHan != NULL);
	if (toret)
	{
		::TerminateThread(RemoveneThreadHan, 0);
		RemoveneThreadHan = NULL;
		//Activation du rafraichissement
		if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
			PlaylistDlg->m_playlistat.SetRedraw(TRUE);
	}
	return toret;
}

void CPlaylistDlg::OnArrangeRemovene()
{
	if (RemoveneThreadHan == NULL)
	{
		RemoveneThreadHan = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)RemoveneThread, NULL, 0, &RemoveneThreadID);
		SetThreadPriority(RemoveneThreadHan, THREAD_PRIORITY_LOWEST);
	}
}

unsigned long __stdcall RemoveDuplicateEntriesThread()
{
	int i = 0, j;

	//Dsactivation du rafraichissement
	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		PlaylistDlg->m_playlistat.SetRedraw(FALSE);

	//Tri d'abord
	m_PlaylistManager.QuickSort(TRUE);

	while (i < m_PlaylistManager.GetSize() - 1 && m_PlaylistManager.GetSize() > 0)
	{
		j = i + 1;
		while (m_PlaylistManager.GetPlaylist(i) == m_PlaylistManager.GetPlaylist(j))
		{
			m_PlaylistManager.RemoveAt(j);
			if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
				PlaylistDlg->m_playlistat.DeleteString(j);

			if ((UINT)j < pMainFrame->Current)
				pMainFrame->Current--;
			else if ((UINT)j == pMainFrame->Current)
				pMainFrame->Current = i;
		}
		i++;
	}

	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
	{
		PlaylistDlg->m_playlistat.SetSel(-1, FALSE);
		PlaylistDlg->ShowFileName();
		//Activation du rafraichissement
		PlaylistDlg->m_playlistat.SetRedraw(TRUE);
		PlaylistDlg->m_playlistat.Invalidate();
	}
	PlaylistDlg->RemoveDuplicateEntriesThreadHan = NULL;
	return TRUE;
}

bool CPlaylistDlg::StopRemoveDuplicateEntries()
{
	bool toret = (RemoveDuplicateEntriesThreadHan != NULL);
	if (toret)
	{
		::TerminateThread(RemoveDuplicateEntriesThreadHan, 0);
		RemoveDuplicateEntriesThreadHan = NULL;
		//Activation du rafraichissement
		if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
			PlaylistDlg->m_playlistat.SetRedraw(TRUE);
	}
	return toret;
}

void CPlaylistDlg::OnArrangeRemoveDuplicateEntries()
{
	if (RemoveDuplicateEntriesThreadHan == NULL)
	{
		RemoveDuplicateEntriesThreadHan = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)RemoveDuplicateEntriesThread, NULL, 0, &RemoveDuplicateEntriesThreadID);
		SetThreadPriority(RemoveDuplicateEntriesThreadHan, THREAD_PRIORITY_LOWEST);
	}
}

bool CPlaylistDlg::StopSort()
{
	bool toret = (SortThreadHan != NULL);
	if (toret)
	{
		::TerminateThread(SortThreadHan, 0);
		SortThreadHan = NULL;
		//Activation du rafraichissement
		if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
			PlaylistDlg->m_playlistat.SetRedraw(TRUE);
	}
	return toret;
}

unsigned long __stdcall SortThread()
{
	if (m_PlaylistManager.GetSize() == 0)
	{
		PlaylistDlg->SortThreadHan = NULL;
		return TRUE;
	}
	//Dsactivation du rafraichissement
	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		PlaylistDlg->m_playlistat.SetRedraw(FALSE);

	m_PlaylistManager.QuickSort(TRUE);

	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
	{
		PlaylistDlg->m_playlistat.SetSel(-1, FALSE);
		PlaylistDlg->ShowFileName();
		//Activation du rafraichissement
		PlaylistDlg->m_playlistat.SetRedraw(TRUE);
		PlaylistDlg->m_playlistat.Invalidate();
	}
	PlaylistDlg->SortThreadHan = NULL;
	return TRUE;
}
void CPlaylistDlg::OnArrangeSort()
{
	if (SortThreadHan == NULL)
	{
		SortThreadHan = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)SortThread, NULL, 0, &SortThreadID);
		SetThreadPriority(SortThreadHan, THREAD_PRIORITY_LOWEST);
	}
}
void CPlaylistDlg::OnArrangeSortByFilename()
{
	if (SortThreadHan == NULL)
	{
		SortThreadHan = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)SortThread, NULL, 0, &SortThreadID);
		SetThreadPriority(SortThreadHan, THREAD_PRIORITY_LOWEST);
	}
}

void CPlaylistDlg::OnCancel()
{
	SaveWindowPosAndDestroy();
}

void CPlaylistDlg::OnChangeSearch()
{
	CString buf, buf2;
	GetDlgItem(IDC_SEARCH)->GetWindowText(buf);
	if (buf == "")
		return;

	UINT z = _ttoi(buf);
	if (z > 0 && z <= (UINT)m_PlaylistManager.GetSize())
	{
		m_playlistat.SetSel(-1, FALSE);
		m_playlistat.SetSel(z - 1, TRUE);
		m_playlistat.SetCaretIndex(z - 1, TRUE);
		m_playlistat.SetTopIndex(z - 1);
		ShowFileName();
		return;
	}

	buf.MakeLower();
	int q = -1;
	for (UINT i = 0; i < (UINT)m_PlaylistManager.GetSize(); i++)
	{
		buf2 = m_PlaylistManager.GetPlaylist(i);
		buf2.MakeLower();
		if (buf2.Find(buf) > -1)
		{
			q = i;
			break;
		}
	}
	if (q != -1)
	{
		m_playlistat.SetSel(-1, FALSE);
		m_playlistat.SetSel(q, TRUE);
		m_playlistat.SetCaretIndex(q, TRUE);
		m_playlistat.SetTopIndex(q);
		ShowFileName();
	}
}

void CPlaylistDlg::OnClose()
{
	SaveWindowPosAndDestroy();
}

void CPlaylistDlg::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	if (!CMenuXP::OnDrawItem(lpDrawItemStruct, m_hWnd) || !pMainFrame->XPLookNFeel)
		CDialog::OnDrawItem(nIDCtl, lpDrawItemStruct);
}   																		   
void CPlaylistDlg::OnDropFiles(HDROP hDropInfo)
{
	UINT PLLPrev = (UINT)m_PlaylistManager.GetSize();

	TCHAR szFileName[_MAX_PATH];
	CString buf;
	UINT nFileCount = ::DragQueryFile(hDropInfo, 0xFFFFFFFF, NULL, 0);
	for (UINT i = 0; i < nFileCount; i++)
	{
		::DragQueryFile(hDropInfo, i, szFileName, _MAX_PATH);
		buf = szFileName;
		if (::GetFileAttributes(szFileName) & FILE_ATTRIBUTE_DIRECTORY)
		{
			if (buf[buf.GetLength() - 1] != '\\')
				buf += "\\";
			FindMP3Files(buf);
		}
		else
		{
			if ((buf.Right(4)).CompareNoCase(_T(".M3U")) == 0 || (buf.Right(4)).CompareNoCase(_T(".PLS")) == 0)
				pMainFrame->OpenPlaylist(buf, false, false);
			else
			{
				m_PlaylistManager.Add(buf);
				m_playlistat.AddString(_T(""));
			}
		}
	}

	m_PlaylistManager.QuickSort(TRUE);

	if (m_PlaylistManager.GetSize() > 0)
	{
		if (PLLPrev == 0)
		{
			if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
				::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
			if (pMainFrame->ShuffleMode)
				pMainFrame->FirstInMyRandom = pMainFrame->Current = pMainFrame->MyRandom(true, true);
			else
			{
				pMainFrame->Current = 0;pMainFrame->FirstInMyRandom = 0xFFFFFFFF;
			}
			bool first = true;
			int q = 0;
			do
			{
				if (!first)
				{
					if (pMainFrame->Current < (UINT)m_PlaylistManager.GetSize() - 1)
						pMainFrame->Current++;
					else
					{
						pMainFrame->Current = 0;
						q++;
						if (pMainFrame->m_Repeat != 1)
							break;
					}
				}
				else
					first = false;
				if (pMainFrame->AutoPlay)
				{
					enqueueFileWithMetaStruct eFile;
					eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
					::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
				}
				else if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
					::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
				break;
			}
			while (q < 2);
		}
		m_PlaylistManager.ReadTagAT(pMainFrame->Current);
		pMainFrame->UpdMBTR();
	}
	ShowFileName();
	m_playlistat.Invalidate();

	::DragFinish(hDropInfo);
}

void CPlaylistDlg::OnEnterMenuLoop(BOOL bIsTrackPopupMenu)
{
	if (!bIsTrackPopupMenu)
	{
		CMenu* pPopup;
		pPopup = GetMenu()->GetSubMenu(3);
		while (pPopup->DeleteMenu(0, MF_BYPOSITION));
		if (m_PlaylistManager.GetSize() > 0 && m_PlaylistManager.GetSize() <= 50000)
			pMainFrame->BuildMenu(pPopup);
	}
	CDialog::OnEnterMenuLoop(bIsTrackPopupMenu);
}

void CPlaylistDlg::OnFileAddFile()
{
	UINT PLLPrev = m_PlaylistManager.GetSize();
	// open a file
	CFileDialog dialog(TRUE, _T("mp3"), _T(""), OFN_ALLOWMULTISELECT | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, m_Helper.MyLoadString(IDS_OPENFILE), this);

	TCHAR buffer[_MAX_PATH*500 + 1];
	buffer[0] = '\0';
	dialog.m_ofn.lpstrFile = buffer;
	dialog.m_ofn.nMaxFile = _MAX_PATH * 500;
	dialog.m_ofn.lpstrInitialDir = LPCTSTR(pMainFrame->InitDir);
	int ret;
	ret = dialog.DoModal();
	if (ret == IDOK)
	{
		// a file was selected
		POSITION p = dialog.GetStartPosition();
		CString buf;
		while (p != NULL)
		{
			m_playlistat.AddString(_T(""));
			m_PlaylistManager.Add(dialog.GetNextPathName(p));
			pMainFrame->InitDir = buf.Left(buf.ReverseFind('\\') + 1);
		}
		if (PLLPrev == 0)
		{
			if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
				::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
			if (pMainFrame->ShuffleMode)
				pMainFrame->FirstInMyRandom = pMainFrame->Current = pMainFrame->MyRandom(true, true);
			else
				pMainFrame->Current = 0;pMainFrame->FirstInMyRandom = 0xFFFFFFFF;

			bool first = true;
			int q = 0;
			do
			{
				if (!first)
				{
					if (pMainFrame->Current < (UINT)m_PlaylistManager.GetSize() - 1)
						pMainFrame->Current++;
					else
					{
						pMainFrame->Current = 0;
						q++;
						if (pMainFrame->m_Repeat != 1)
							break;
					}
				}
				else
					first = false;
				if (pMainFrame->AutoPlay)
				{
					enqueueFileWithMetaStruct eFile;
					eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
					::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
				}
				else if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
					::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
				break;
			}
			while (q < 2);
		}
		pMainFrame->UpdMBTR();
		ShowFileName();
	}
	m_playlistat.SetFocus();
}

void CPlaylistDlg::OnFileAddFolder()
{
	UINT PLLPrev = m_PlaylistManager.GetSize();

	LPMALLOC pMalloc;
	if (SHGetMalloc(&pMalloc) != NOERROR)
		return;

	BROWSEINFO bi;
	TCHAR buf[MAX_PATH];

	CString title;
	title = m_Helper.MyLoadString(IDS_OPENFOLDER);

	bi.hwndOwner = m_hWnd;
	bi.pszDisplayName = buf;
	bi.pidlRoot = NULL;
	bi.lpszTitle = LPCTSTR(title);
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = BrowseCallbackProc;

	LPITEMIDLIST pidl;
	if ((pidl = SHBrowseForFolder(&bi)) == NULL)
	{
		pMalloc->Free(pidl);
		pMalloc ->Release();
		return;
	}
	if (SHGetPathFromIDList(pidl, buf) == false)
	{
		pMalloc->Free(pidl);
		pMalloc ->Release();
		return;
	}
	pMalloc->Free(pidl);
	pMalloc->Release();

	pMainFrame->OpenFolder(buf, pMainFrame->RandomFirst == 1, true);
}

void CPlaylistDlg::OnFileEditTag()
{
	int i = -1;
	CString buf;
	CTagEditor TagEditor(IDD_TAGEDIT, this);
	while ((i = m_playlistat.GetFirstSelected(i + 1)) > -1)
	{
		if (!FileExists(m_PlaylistManager.GetPlaylist(i)))
			continue;
		TagEditor.m_album = m_PlaylistManager.GetAlbum(i);
		TagEditor.m_artist = m_PlaylistManager.GetArtist(i);
		TagEditor.m_comment = m_PlaylistManager.GetComment(i);
		TagEditor.m_filename = m_PlaylistManager.GetPlaylist(i);
		TagEditor.m_title = m_PlaylistManager.GetTitle(i);
		TagEditor.m_year = m_PlaylistManager.GetYear(i);
		if (TagEditor.DoModal() == IDOK)
		{
			m_PlaylistManager.Add(TagEditor.m_filename, i);

			m_playlistat.RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
			if ((UINT)i == pMainFrame->Current)
			{
				if (MP3InfoDlg != NULL && MP3InfoDlg->m_hWnd != NULL)
				{
					MP3InfoDlg->PrevFile = "*";
					MP3InfoDlg->ReadInfo();
				}
			}
		}
		else
			break;
	}
	SetFocus();
}

void CPlaylistDlg::OnFileRemove()
{
	Remove();
}

void CPlaylistDlg::OnFileSaveInExtendedFormat()
{
	if (pMainFrame->ExtPlaylistFormat)
		pMainFrame->ExtPlaylistFormat = 0;
	else
		pMainFrame->ExtPlaylistFormat = 1;
	GetMenu()->CheckMenuItem(ID_FILE_SAVEINEXTENDEDFORMAT, MF_BYCOMMAND | (pMainFrame->ExtPlaylistFormat ? MF_CHECKED : MF_UNCHECKED));
}

void CPlaylistDlg::OnKillFocusPlaylistAt()
{
	m_playlistat.CtrlDown = m_playlistat.ShiftDown = false;
	m_playlistat.prev = -1;
}

BOOL CPlaylistDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	CMenuXP::UpdateMenuBar(this);
	if (menu != NULL)
		delete menu;
	menu = new CMenu();
	menu->LoadMenu(IDR_PLMENU);
	m_Helper.translateMenu(menu, IDR_PLMENU);
	this->SetMenu(menu);
	m_Helper.translateWindowText(this, IDD_PLAYLIST);

	SetIcon(pMainFrame->StpIcon, TRUE);
	SetIcon(pMainFrame->StpIconSm, FALSE);

	accel = LoadAccelerators(theApp.m_hInstance, MAKEINTRESOURCE(IDR_PLACCEL));

	CRect rect1, rect2;

	GetClientRect(&rect1);

	GetDlgItem(IDC_PLAYLIST_AT)->GetWindowRect(&rect2);
	PlAtH = rect1.Height() - rect2.Height();
	PlAtW = rect1.Width() - rect2.Width();
	GetDlgItem(IDC_FRAME)->GetWindowRect(&rect2);
	FrameH = rect1.Height() - rect2.Height();
	FrameW = rect1.Width() - rect2.Width();
	GetDlgItem(IDC_SEARCH)->GetWindowRect(&rect2);
	SrchH = rect2.Height();
	SrchW = rect1.Width() - rect2.Width();
	GetDlgItem(IDC_FILENAME)->GetWindowRect(&rect2);
	ScreenToClient(&rect2);
	FNameL = rect2.left;
	FNameT = rect1.Height() - rect2.top;
	FNameH = rect2.Height();
	FNameW = rect1.Width() - rect2.Width();

	::SystemParametersInfo(SPI_GETWORKAREA, 0, &rect1, 0);
	GetWindowRect(&rect2);
	if (pMainFrame->PlaylistW == -1000)
		pMainFrame->PlaylistW = rect2.Width();
	if (pMainFrame->PlaylistH == -1000)
		pMainFrame->PlaylistH = rect2.Height();
	if (pMainFrame->PlaylistX == -1000)
		pMainFrame->PlaylistX = (rect1.right - pMainFrame->PlaylistW) / 2;
	if (pMainFrame->PlaylistY == -1000)
		pMainFrame->PlaylistY = (rect1.bottom - pMainFrame->PlaylistH) / 2;
	if (pMainFrame->PlaylistX < 0)
		pMainFrame->PlaylistX = 0;
	if (pMainFrame->PlaylistY < 0)
		pMainFrame->PlaylistY = 0;
	if (pMainFrame->PlaylistX + pMainFrame->PlaylistW > rect1.right)
		pMainFrame->PlaylistX = rect1.right - pMainFrame->PlaylistW;
	if (pMainFrame->PlaylistY + pMainFrame->PlaylistH > rect1.bottom)
		pMainFrame->PlaylistY = rect1.bottom - pMainFrame->PlaylistH;
	SetWindowPos(NULL, pMainFrame->PlaylistX, pMainFrame->PlaylistY, pMainFrame->PlaylistW, pMainFrame->PlaylistH, SWP_NOZORDER | SWP_NOACTIVATE);

	GetClientRect(&rect1);

	GetDlgItem(IDC_PLAYLIST_AT)->SetWindowPos(NULL, 0, 0, rect1.Width() - PlAtW, rect1.Height() - PlAtH, SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
	//   GetDlgItem(IDC_FRAME)->ShowWindow(SW_HIDE);
	GetDlgItem(IDC_FRAME)->SetWindowPos(NULL, 0, 0, rect1.Width() - FrameW, rect1.Height() - FrameH, SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
	//   GetDlgItem(IDC_FRAME)->ShowWindow(SW_SHOW);
	GetDlgItem(IDC_SEARCH)->SetWindowPos(NULL, 0, 0, rect1.Width() - SrchW, SrchH, SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
	GetDlgItem(IDC_FILENAME)->SetWindowPos(NULL, FNameL, rect1.Height() - FNameT, rect1.Width() - FNameW, FNameH, SWP_NOZORDER | SWP_NOACTIVATE);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		pSysMenu->AppendMenu(MF_SEPARATOR);
		pSysMenu->AppendMenu(MF_STRING, 9999, m_Helper.MyLoadString(IDS_ALWAYSONTOP));
	}
	if (pMainFrame->PlaylistOnTop == 1)
	{
		pSysMenu->CheckMenuItem(9999, MF_BYCOMMAND | MF_CHECKED);
		SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
	}
	else
		ModifyStyleEx(0, WS_EX_APPWINDOW);

	FillPlaylist();
	if (m_PlaylistManager.GetSize() > 0)
	{
		m_playlistat.SetSel(pMainFrame->Current, TRUE);
		m_playlistat.SetCaretIndex(pMainFrame->Current, TRUE);
		m_playlistat.SetTopIndex(pMainFrame->Current);
	}
	ShowFileName();

	GetMenu()->CheckMenuItem(ID_FILE_SAVEINEXTENDEDFORMAT, MF_BYCOMMAND | (pMainFrame->ExtPlaylistFormat ? MF_CHECKED : MF_UNCHECKED));

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CPlaylistDlg::OnInitMenuPopup(CMenu* pMenu, UINT wParam, BOOL isActive)
{
	for (UINT i = 0; i < pMenu->GetMenuItemCount(); i++)
	{
		UINT menuID = pMenu->GetMenuItemID(i);
		CString buf, tmp;
		//Si on est pas sur un sous menu
		if (menuID != 0 && menuID != 0xFFFFFFFF)
		{
			pMenu->GetMenuString(i, tmp, MF_BYPOSITION);
			if (tmp.Find(_T("|")) != -1)
			{
				CString sNumber = tmp.Left(tmp.Find(_T("|")));
				int number = 0;
				for (int j = 0; j < sNumber.GetLength(); j++)
					number = number * 10 + sNumber.GetAt(j) - '0';
				buf = tmp.Mid(tmp.ReverseFind('\\') + 1, tmp.GetLength() - tmp.ReverseFind('\\') - 5);
				if (m_PlaylistManager.GetTag(number).GetLength() == 0)
					m_PlaylistManager.ReadTagAT(number);
				pMenu->ModifyMenu(menuID, MF_BYCOMMAND | pMenu->GetMenuState(menuID, MF_BYCOMMAND), menuID, m_PlaylistManager.GetTag(number));
			}
		}
	}
	CDialog::OnInitMenuPopup(pMenu, wParam, isActive);
	CMenuXP::SetXPLookNFeel(this, pMenu->m_hMenu, pMainFrame->XPLookNFeel);
}

void CPlaylistDlg::OnMeasureItem(int, LPMEASUREITEMSTRUCT lpMeasureItemStruct)
{
	CMenuXP::OnMeasureItem(lpMeasureItemStruct);
}   																		   
LRESULT CPlaylistDlg::OnMenuChar(UINT nChar, UINT nFlags, CMenu* pMenu)
{
	if (CMenuXP::IsOwnerDrawn(pMenu->m_hMenu))
		return CMenuXP::OnMenuChar(pMenu->m_hMenu, nChar, nFlags);  																	   
	return CDialog::OnMenuChar(nChar, nFlags, pMenu);
}

void CPlaylistDlg::OnOK()
{
	if (GetFocus()->GetDlgCtrlID() == IDC_PLAYLIST_AT || GetFocus()->GetDlgCtrlID() == IDC_SEARCH)
	{
		if (m_playlistat.GetFirstSelected() != -1)
		{
			PlayTheFile(m_playlistat.GetFirstSelected());
			m_playlistat.RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
		}
	}
}

void CPlaylistDlg::OnPlaylistOpen()
{
	// open a file
	CFileDialog dialog(TRUE, _T("m3u"), _T(""), OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, m_Helper.MyLoadString(IDS_OPENPLAYLIST), this);
	dialog.m_ofn.lpstrInitialDir = LPCTSTR(pMainFrame->InitDir);
	int ret;
	ret = dialog.DoModal();
	if (ret == IDOK)
	{
		pMainFrame->OpenPlaylist(dialog.GetPathName());
		pMainFrame->InitDir = dialog.GetPathName().Left(dialog.GetPathName().ReverseFind('\\') + 1);
	}
	m_playlistat.SetFocus();
}

void CPlaylistDlg::OnPlaylistSave()
{
	PlaylistSave();
}


void CPlaylistDlg::OnPopupSelectAll()
{
	m_playlistat.SetCaretIndex(0);
	m_playlistat.SetSel(-1, TRUE);
	ShowFileName();
}

void CPlaylistDlg::OnSize(UINT nType, int cx, int cy)
{
	CDialog::OnSize(nType, cx, cy);
	if (CanResize)
	{
		m_playlistat.memDC.DeleteDC();
		m_playlistat.memBitmap.DeleteObject();
		m_playlistat.MemDCCreated = false;

		CRect rect;
		GetClientRect(&rect);

		GetDlgItem(IDC_PLAYLIST_AT)->SetWindowPos(NULL, 0, 0, rect.Width() - PlAtW, rect.Height() - PlAtH, SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
		//GetDlgItem(IDC_FRAME)->ShowWindow(SW_HIDE);
		GetDlgItem(IDC_FRAME)->SetWindowPos(NULL, 0, 0, rect.Width() - FrameW, rect.Height() - FrameH, SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
		//GetDlgItem(IDC_FRAME)->ShowWindow(SW_SHOW);
		GetDlgItem(IDC_SEARCH)->SetWindowPos(NULL, 0, 0, rect.Width() - SrchW, SrchH, SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
		GetDlgItem(IDC_FILENAME)->SetWindowPos(NULL, FNameL, rect.Height() - FNameT, rect.Width() - FNameW, FNameH, SWP_NOZORDER | SWP_NOACTIVATE);
	}
}

void CPlaylistDlg::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CDialog::OnShowWindow(bShow, nStatus);

	CanResize = bShow ? true : false;
}

void CPlaylistDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if (nID == 9999)
	{
		if (pMainFrame->PlaylistOnTop == 1)
		{
			GetSystemMenu(FALSE)->CheckMenuItem(9999, MF_BYCOMMAND | MF_UNCHECKED);

			ShowWindow(SW_HIDE);
			ModifyStyleEx(0, WS_EX_APPWINDOW);
			ShowWindow(SW_SHOW);

			SetWindowPos(&wndNoTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);

			pMainFrame->PlaylistOnTop = 0;
		}
		else
		{
			GetSystemMenu(FALSE)->CheckMenuItem(9999, MF_BYCOMMAND | MF_CHECKED);

			ShowWindow(SW_HIDE);
			ModifyStyleEx(WS_EX_APPWINDOW, 0);
			ShowWindow(SW_SHOW);

			SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);

			pMainFrame->PlaylistOnTop = 1;
		}
		if (SeekDialog != NULL && SeekDialog->m_hWnd != NULL)
			SeekDialog->SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
	}
	else
		CDialog::OnSysCommand(nID, lParam);
}

void CPlaylistDlg::OnTrackMenu(UINT nID)
{
	m_playlistat.SetSel(-1, FALSE);
	m_playlistat.SetSel(nID - IDM_TRACKS, TRUE);
	m_playlistat.SetCaretIndex(nID - IDM_TRACKS, TRUE);
	m_playlistat.SetTopIndex(nID - IDM_TRACKS);
	ShowFileName();
}

void CPlaylistDlg::OnTracksAddPlaylist()
{
	// open a file
	CFileDialog dialog(TRUE, _T("m3u"), _T(""), OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, m_Helper.MyLoadString(IDS_OPENPLAYLIST), this);
	dialog.m_ofn.lpstrInitialDir = LPCTSTR(pMainFrame->InitDir);
	int ret;
	ret = dialog.DoModal();
	if (ret == IDOK)
	{
		pMainFrame->OpenPlaylist(dialog.GetPathName(), false, true);
		pMainFrame->InitDir = dialog.GetPathName().Left(dialog.GetPathName().ReverseFind('\\') + 1);
		ShowFileName();
	}
	m_playlistat.SetFocus();
}

void CPlaylistDlg::OnTracksFileManagementDelete()
{
	if (MessageBox(m_Helper.MyLoadString(IDS_DELETEFILE), _T("STP"), MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
		Remove(false, true);
}

void CPlaylistDlg::OnTracksFileManagementCopyTo()
{
	LPMALLOC pMalloc;
	if (SHGetMalloc(&pMalloc) != NOERROR)
		return;

	BROWSEINFO bi;
	TCHAR buf[MAX_PATH];

	CString title;
	title = m_Helper.MyLoadString(IDS_TARGETFOLDER);

	bi.hwndOwner = m_hWnd;
	bi.pszDisplayName = buf;
	bi.pidlRoot = NULL;
	bi.lpszTitle = LPCTSTR(title);
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = BrowseCallbackProc;

	LPITEMIDLIST pidl;
	if ((pidl = SHBrowseForFolder(&bi)) == NULL)
	{
		pMalloc->Free(pidl);
		pMalloc ->Release();
		return;
	}
	if (SHGetPathFromIDList(pidl, buf) == false)
	{
		pMalloc->Free(pidl);
		pMalloc ->Release();
		return;
	}
	pMalloc->Free(pidl);
	pMalloc->Release();

	CString theDir = buf;
	if (theDir[theDir.GetLength() - 1] != '\\')
		theDir += "\\";

	pMainFrame->InitDir = theDir;

	CDialog dlg;
	dlg.Create(IDD_PROGRESS, this);
	CRect rect1, rect2;
	GetWindowRect(&rect1);
	dlg.GetWindowRect(&rect2);
	dlg.SetWindowPos(NULL, rect1.left + (rect1.right - rect1.left - (rect2.right - rect2.left)) / 2, rect1.top + (rect1.bottom - rect1.top - (rect2.bottom - rect2.top)) / 2, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
	dlg.ShowWindow(SW_SHOW);
	dlg.UpdateWindow();

	((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetRange(0, m_playlistat.GetSelCount());
	((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetStep(1);

	CString srcfile, targetfile;
	int i = -1;
	while ((i = m_playlistat.GetFirstSelected(i + 1)) > -1)
	{
		srcfile = m_PlaylistManager.GetPlaylist(i);
		targetfile = theDir + srcfile.Mid(srcfile.ReverseFind('\\') + 1);
		dlg.SetDlgItemText(IDC_FNAME, srcfile.Mid(srcfile.ReverseFind('\\') + 1));
		if (FileExists(targetfile))
		{
			if (dlg.MessageBox(m_Helper.MyLoadString(IDS_OVERWRITE) + _T("\n") + targetfile + _T("?"), _T("STP"), MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
			{
				::SetFileAttributes(targetfile, ::GetFileAttributes(targetfile) & !FILE_ATTRIBUTE_READONLY);
				::DeleteFile(targetfile);
				CopyFile(srcfile, targetfile, TRUE);
			}
		}
		else
			CopyFile(srcfile, targetfile, TRUE);
		((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->StepIt();
	}
}

void CPlaylistDlg::OnTracksFileManagementMoveTo()
{
	LPMALLOC pMalloc;
	if (SHGetMalloc(&pMalloc) != NOERROR)
		return;

	BROWSEINFO bi;
	TCHAR buf[MAX_PATH];

	CString title;
	title = m_Helper.MyLoadString(IDS_TARGETFOLDER);

	bi.hwndOwner = m_hWnd;
	bi.pszDisplayName = buf;
	bi.pidlRoot = NULL;
	bi.lpszTitle = LPCTSTR(title);
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.lpfn = BrowseCallbackProc;

	LPITEMIDLIST pidl;
	if ((pidl = SHBrowseForFolder(&bi)) == NULL)
	{
		pMalloc->Free(pidl);
		pMalloc ->Release();
		return;
	}
	if (SHGetPathFromIDList(pidl, buf) == false)
	{
		pMalloc->Free(pidl);
		pMalloc ->Release();
		return;
	}
	pMalloc->Free(pidl);
	pMalloc->Release();

	CString theDir = buf;
	if (theDir[theDir.GetLength() - 1] != '\\')
		theDir += "\\";

	pMainFrame->InitDir = theDir;

	CDialog dlg;
	dlg.Create(IDD_PROGRESS, this);
	CRect rect1, rect2;
	GetWindowRect(&rect1);
	dlg.GetWindowRect(&rect2);
	dlg.SetWindowPos(NULL, rect1.left + (rect1.right - rect1.left - (rect2.right - rect2.left)) / 2, rect1.top + (rect1.bottom - rect1.top - (rect2.bottom - rect2.top)) / 2, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
	dlg.ShowWindow(SW_SHOW);
	dlg.UpdateWindow();

	((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetRange(0, m_playlistat.GetSelCount());
	((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->SetStep(1);

	CString srcfile, targetfile;
	int i = -1, tdi = 0, curpos = 0;
	while ((i = m_playlistat.GetFirstSelected(i + 1)) > -1)
	{
		srcfile = m_PlaylistManager.GetPlaylist(i);
		targetfile = theDir + srcfile.Mid(srcfile.ReverseFind('\\') + 1);
		dlg.SetDlgItemText(IDC_FNAME, srcfile.Mid(srcfile.ReverseFind('\\') + 1));
		if (FileExists(targetfile))
		{
			if (dlg.MessageBox(m_Helper.MyLoadString(IDS_OVERWRITE) + _T("\n") + targetfile + _T("?"), _T("STP"), MB_YESNO | MB_ICONEXCLAMATION | MB_DEFBUTTON2) == IDYES)
			{
				::SetFileAttributes(targetfile, ::GetFileAttributes(targetfile) & !FILE_ATTRIBUTE_READONLY);
				::DeleteFile(targetfile);
				if ((UINT)i == pMainFrame->Current)
				{
					curpos = stdIn->GetOutputTime() / 1000;
					if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
						::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
				}

				if (MoveFile(srcfile, targetfile))
					m_PlaylistManager.Add(targetfile, i);

				if ((UINT)i == pMainFrame->Current)
				{
					enqueueFileWithMetaStruct eFile;
					eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
					::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
					if (tdi != 0)
						stdIn->SetOutputTime(curpos * 1000);
					if (stdIn->IsPaused())
						::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON3, 0);
				}
			}
		}
		else
		{
			if ((UINT)i == pMainFrame->Current)
			{
				curpos = stdIn->GetOutputTime() / 1000;
				if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
					::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
			}

			if (MoveFile(srcfile, targetfile))
				m_PlaylistManager.Add(targetfile, i);

			if ((UINT)i == pMainFrame->Current)
			{
				enqueueFileWithMetaStruct eFile;
				eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
				::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
				if (tdi != 0)
					stdIn->SetOutputTime(curpos * 1000);
				if (stdIn->IsPaused())
					::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON3, 0);
			}
		}
		((CProgressCtrl *)dlg.GetDlgItem(IDC_PROGRESS))->StepIt();
	}
	ShowFileName();
}

void CPlaylistDlg::OnTracksRemoveAll()
{
	Remove(true);
}

void CPlaylistDlg::OnTracksRenameFiles()
{
	UINT i = 0xFFFFFFFF;
	int tdi = 0, curpos = 0, j;
	if (m_playlistat.GetFirstSelected() == -1)
		return;

	CRenameDlg dlg(IDD_RENAME, this);
	dlg.m_template = pMainFrame->RNTemplate;

	if (dlg.DoModal() == IDOK)
	{
		pMainFrame->RNTemplate = dlg.m_template;

		CFile mpfile;

		CString track, artist, title, album, fname, albtrack;

		char buf[31];
		while ((i = m_playlistat.GetFirstSelected(i + 1)) > -1)
		{
			if (mpfile.Open(m_PlaylistManager.GetPlaylist(i), CFile::modeRead | CFile::shareDenyNone))
			{
				track.Format(_T("%02d"), i + 1);
				UINT b, e;
				pMainFrame->GetAlbumRange(&b, &e, true, i);
				albtrack.Format(_T("%02d"), i - b + 1);

				if (mpfile.GetLength() < 129)
				{
					mpfile.Close();
					title = "Track" + albtrack;
					artist = "Unknown artist";
					album = "Unknown album";
				}
				else
				{
					mpfile.Seek(-128, CFile::end);
					mpfile.Read(buf, 3);buf[3] = '\0';
					if (strcmp(buf, "TAG"))
					{
						mpfile.Close();
						title = "Track" + albtrack;
						artist = "Unknown artist";
						album = "Unknown album";
					}
					else
					{
						if (mpfile.Read(buf, 30))
						{
							buf[30] = '\0';
							title = buf;
							title.TrimRight();
							if (title == "")
								title = "Track" + albtrack;
						}
						if (mpfile.Read(buf, 30))
						{
							buf[30] = '\0';
							artist = buf;
							artist.TrimRight();
							if (artist == "")
								artist = "Unknown artist";
						}
						if (mpfile.Read(buf, 30))
						{
							buf[30] = '\0';
							album = buf;
							album.TrimRight();
							if (album == "")
								album = "Unknown album";
						}
						mpfile.Close();
					}
				}

				fname = pMainFrame->RNTemplate;
				fname.Replace(_T("%1"), track);
				fname.Replace(_T("%2"), artist);
				fname.Replace(_T("%3"), title);
				fname.Replace(_T("%4"), album);
				fname.Replace(_T("%5"), albtrack);

				j = m_PlaylistManager.GetPlaylist(i).ReverseFind('.');
				if (j >= 0)
					fname += m_PlaylistManager.GetPlaylist(i).Mid(j);

				fname.Replace('\\', '_');
				fname.Replace('/', '_');
				fname.Replace(':', '_');
				fname.Replace('*', '_');
				fname.Replace('?', '_');
				fname.Replace('\"', '_');
				fname.Replace('<', '_');
				fname.Replace('>', '_');
				fname.Replace('|', '_');

				fname = m_PlaylistManager.GetPlaylist(i).Left(m_PlaylistManager.GetPlaylist(i).ReverseFind('\\') + 1) + fname;

				if ((UINT)i == pMainFrame->Current)
				{
					curpos = stdIn->GetOutputTime() / 1000;

					if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
						::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
				}

				if (::MoveFile(m_PlaylistManager.GetPlaylist(i), fname))
					m_PlaylistManager.Add(fname, i);

				if ((UINT)i == pMainFrame->Current)
				{
					enqueueFileWithMetaStruct eFile;
					eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
					::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
					if (tdi != 0)
						stdIn->SetOutputTime(1000 * curpos);
					if (stdIn->IsPaused())
						::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON3, 0);
				}
			}
		}
		ShowFileName();
		pMainFrame->UpdMBTR();
	}
}

void CPlaylistDlg::OnTracksReReadAllTags()
{
	m_PlaylistManager.StopReadAll();
	UINT i;
	for (i = 0; i < (UINT)m_PlaylistManager.GetSize(); i++)
	{
		if (i != pMainFrame->Current)
			m_PlaylistManager.Add(m_PlaylistManager.Get(i), i);
	}
	m_PlaylistManager.ReadAllTags();
}

void CPlaylistDlg::OnTracksSelectionAddToExistentPlaylist()
{
	CFileDialog dialog(TRUE, _T("m3u"), _T(""), OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, m_Helper.MyLoadString(IDS_OPENPLAYLIST), this);
	dialog.m_ofn.lpstrInitialDir = LPCTSTR(pMainFrame->InitDir);
	int ret;
	ret = dialog.DoModal();
	if (ret == IDOK)
	{
		pMainFrame->InitDir = dialog.GetPathName().Left(dialog.GetPathName().ReverseFind('\\') + 1);

		CStringArray CurPL;
		CString fname = dialog.GetPathName();
		CStdioFile plfile;

		if (!plfile.Open(fname, CFile::modeRead | CFile::shareDenyNone))
		{
			MessageBox(m_Helper.MyLoadString(IDS_CANTOPEN), _T("STP"), MB_ICONERROR);
			return;
		}

		CString buf;
		CString CurPath = fname.Left(fname.ReverseFind('\\') + 1);
		CString CurDisk = fname.Left(fname.Find(':') + 1);
		while (plfile.ReadString(buf))
		{
			buf.TrimLeft();buf.TrimRight();
			if (buf.Left(4) == "#EXT" || buf == "" || buf.Left(5) == "#TAG=")
				continue;
			if (buf[1] == ':' || buf.Left(7).CompareNoCase(_T("http://")) == 0 || buf.Left(6).CompareNoCase(_T("ftp://")) == 0)
				CurPL.Add(buf);
			else if (buf[0] == '\\')
				CurPL.Add(CurDisk + buf);
			else
				CurPL.Add(CurPath + buf);
		}
		plfile.Close();
		if (!plfile.Open(dialog.GetPathName(), CFile::modeCreate | CFile::modeWrite))
		{
			MessageBox(m_Helper.MyLoadString(IDS_CANTOPEN), _T("STP"), MB_ICONERROR);
			return;
		}

		UINT i;
		for (i = 0; i < (UINT)m_PlaylistManager.GetSize(); i++)
		{
			if (m_playlistat.GetSel(i))
				CurPL.Add(m_PlaylistManager.GetPlaylist(i));
		}

		CurPath = CurPath.Left(CurPath.GetLength() - 1);
		UINT ln = CurPL.GetUpperBound() + 1;
		for (i = 0; i < ln; i++)
		{
			buf = CurPL[i];
			if (CurPath.CompareNoCase(buf.Left(CurPath.GetLength())) == 0)
				buf = buf.Mid(CurPath.GetLength() + 1);
			if (CurPath.Left(2).CompareNoCase(buf.Left(2)) == 0)
				buf = buf.Mid(2);
			plfile.WriteString(buf + "\n");
		}

		plfile.Close();
	}
}

void CPlaylistDlg::OnTracksSelectionCreateNewPlaylist()
{
	PlaylistSave(false);
}


void CPlaylistDlg::PlaylistSave(bool saveall)
{
	if (m_PlaylistManager.GetSize() == 0)
		return;
	if (!saveall && m_playlistat.GetFirstSelected() == -1)
		return;
	CFileDialog dialog(FALSE, _T("m3u"), _T(""), OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, m_Helper.MyLoadString(IDS_OPENPLAYLIST), this);
	dialog.m_ofn.lpstrInitialDir = LPCTSTR(pMainFrame->InitDir);
	int ret;
	ret = dialog.DoModal();
	if (ret == IDOK)
	{
		// a file was selected
		pMainFrame->InitDir = dialog.GetPathName().Left(dialog.GetPathName().ReverseFind('\\') + 1);
		CStdioFile plfile;
		if (plfile.Open(dialog.GetPathName(), CFile::modeCreate | CFile::modeWrite))
		{
			CString PLPath;
			PLPath = dialog.GetPathName().Left(dialog.GetPathName().ReverseFind('\\'));
			UINT i;
			CString buf;
			for (i = 0; i < (UINT)m_PlaylistManager.GetSize(); i++)
			{
				if (m_playlistat.GetSel(i) || saveall)
				{
					buf = m_PlaylistManager.GetPlaylist(i);
					if (PLPath.CompareNoCase(buf.Left(PLPath.GetLength())) == 0)
						buf = buf.Mid(PLPath.GetLength() + 1);
					if (PLPath.Left(2).CompareNoCase(buf.Left(2)) == 0)
						buf = buf.Mid(2);
					plfile.WriteString(buf + "\n");
				}
			}
			plfile.Close();
			pMainFrame->AddRecent(dialog.GetPathName());
		}
		else
			MessageBox(m_Helper.MyLoadString(IDS_CANTOPEN), _T("STP"), MB_ICONERROR);
	}    
	m_playlistat.SetFocus();
}

void CPlaylistDlg::PlayTheFile(int fnum)
{
	pMainFrame->Current = fnum;
	if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
		::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);

	bool first = true;
	int q = 0;
	do
	{
		if (!first)
		{
			if (pMainFrame->Current < (UINT)m_PlaylistManager.GetSize() - 1)
				pMainFrame->Current++;
			else
			{
				pMainFrame->Current = 0;
				q++;
				if (pMainFrame->m_Repeat != 1)
					break;
			}
		}
		else
			first = false;
		enqueueFileWithMetaStruct eFile;
		eFile.filename = (TCHAR *)(LPCTSTR)m_PlaylistManager.GetPlaylist(pMainFrame->Current);
		::SendMessage(WAEmuWnd->m_hWnd, IPC_PLAYFILE, 0, (WPARAM)&eFile);
		break;
	}
	while (q < 2);
	pMainFrame->UpdMBTR();
}

BOOL CPlaylistDlg::PreTranslateMessage(MSG* pMsg)
{
	int q = TranslateAccelerator(m_hWnd, accel, pMsg);
	if (q > 0)
		return q;
	else
		return CDialog::PreTranslateMessage(pMsg);
}

void CPlaylistDlg::Remove(bool RAll, bool FromDisk)
{
	int q;
	if ((q = m_playlistat.GetFirstSelected()) == -1 && !RAll)
		return;
	int i = 0;
	bool CurDeleted = false;
	while (i < m_playlistat.GetCount() && m_playlistat.GetCount() > 0)
	{
		if (m_playlistat.GetSel(i) || RAll)
		{
			m_playlistat.DeleteString(i);
			if ((UINT)i < pMainFrame->Current)
				pMainFrame->Current--;
			else if ((UINT)i == pMainFrame->Current && !CurDeleted)
			{
				CurDeleted = true;
				if (::SendMessage(WAEmuWnd->m_hWnd, WM_WA_IPC, 0, IPC_ISPLAYING) != PLAYER_MODE_CLOSE)
					::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
			}
			if (FromDisk)
			{
				::SetFileAttributes(m_PlaylistManager.GetPlaylist(i), ::GetFileAttributes(m_PlaylistManager.GetPlaylist(i)) & !FILE_ATTRIBUTE_READONLY);
				::DeleteFile(m_PlaylistManager.GetPlaylist(i));
			}
			m_PlaylistManager.RemoveAt(i);
		}
		else
			i++;
	}
	if (CurDeleted)
	{
		pMainFrame->Current = 0;
		if (m_PlaylistManager.GetSize() == 0)
		{
			CRect rect;
			if (MP3InfoDlg != NULL && MP3InfoDlg->m_hWnd != NULL)
				MP3InfoDlg->SaveWindowPosAndDestroy();
		}
		pMainFrame->UpdMBTR();
	}
	if (m_playlistat.SetSel(q, TRUE) == LB_ERR)
		m_playlistat.SetSel(q - 1, TRUE);
	ShowFileName();
	m_playlistat.RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
}

void CPlaylistDlg::SaveWindowPosAndDestroy()
{
	DestroyAcceleratorTable(accel);

	ShowWindow(SW_RESTORE);
	CRect rect;
	GetWindowRect(&rect);
	pMainFrame->PlaylistX = rect.left;
	pMainFrame->PlaylistY = rect.top;
	pMainFrame->PlaylistW = rect.Width();
	pMainFrame->PlaylistH = rect.Height();

	DestroyWindow();
}

void CPlaylistDlg::ShowFileName()
{
	ShowTimes();

	int i;
	if ((i = m_playlistat.GetFirstSelected()) > -1)
	{
		m_filename.SetWindowText(m_PlaylistManager.GetPlaylist(i));
		m_filename.SetSel(m_PlaylistManager.GetPlaylist(i).GetLength());
	}
	else
		m_filename.SetWindowText(_T(""));
	GetMenu()->EnableMenuItem(ID_PLAYLIST_SAVE, m_PlaylistManager.GetSize() == 0 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_FILE_REMOVE, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_TRACKS_REMOVEALL, m_PlaylistManager.GetSize() < 1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_FILE_EDITTAG, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_TRACKS_REREADALLTAGS, m_PlaylistManager.GetSize() < 1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_TRACKS_SELECTION_CREATENEWPLAYLIST, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_TRACKS_SELECTION_ADDTOEXISTENTPLAYLIST, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_ARRANGE_SORT, m_PlaylistManager.GetSize() == 0 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_ARRANGE_SORTBYFILENAME, m_PlaylistManager.GetSize() == 0 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_ARRANGE_SHUFFLE, m_PlaylistManager.GetSize() == 0 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_TRACKS_RENAMEFILES, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_TRACKS_FILEMANAGEMENT_DELETE, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_TRACKS_FILEMANAGEMENT_COPYTO, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	GetMenu()->EnableMenuItem(ID_TRACKS_FILEMANAGEMENT_MOVETO, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
}

void CPlaylistDlg::ShowTimes()
{
	UINT selected = 0, j = 0, k = 0, q, TotalTime = 0;

	CString buf;
	for (q = 0; q < (UINT)m_PlaylistManager.GetSize(); q++)
	{
		TotalTime += m_PlaylistManager.GetDurationI(q);
		if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		{
			if (m_playlistat.GetSel(q))
				selected += m_PlaylistManager.GetDurationI(q);
		}
	}

	if (TotalTime >= 3600)
		buf.Format(_T("%02d:%02d:%02d / %02d:%02d:%02d"), selected / 3600, (selected % 3600) / 60, selected % 60, TotalTime / 3600, (TotalTime % 3600) / 60, TotalTime % 60);
	else
		buf.Format(_T("%02d:%02d / %02d:%02d"), selected / 60, selected % 60, TotalTime / 60, TotalTime % 60);
	if (PlaylistDlg != NULL && PlaylistDlg->m_hWnd != NULL)
		SetWindowText(m_Helper.MyLoadString(IDS_PLAYLIST) + " " + buf);
}

CPLLB::CPLLB() : CListBox()
{
	menu = new CMenu();
	MemDCCreated = false;
	ShiftDown = false;
	CtrlDown = false;
	prev = -1;
}
CPLLB::~CPLLB()
{
	delete menu;
}

void CPLLB::DrawItem(LPDRAWITEMSTRUCT lpDIS)
{
	if ((int)lpDIS->itemID < 0)
		return; 

	CDC* pDC = CDC::FromHandle(lpDIS->hDC);

	COLORREF crText;
	CString sText;

	if (lpDIS->itemAction & (ODA_SELECT | ODA_DRAWENTIRE))
	{
		if (lpDIS->itemState & ODS_SELECTED)
		{
			CBrush brush(::GetSysColor(COLOR_HIGHLIGHT));
			pDC->FillRect(&lpDIS->rcItem, &brush);
		}
		else
		{
			CBrush brush(::GetSysColor(COLOR_WINDOW));
			pDC->FillRect(&lpDIS->rcItem, &brush);
		}
	}   	

	/*if ((lpDIS->itemAction & ODA_FOCUS) && (lpDIS->itemState & ODS_FOCUS))
	   pDC->DrawFocusRect(&lpDIS->rcItem); 
	if ((lpDIS->itemAction & ODA_FOCUS) &&     !(lpDIS->itemState & ODS_FOCUS))
	   pDC->DrawFocusRect(&lpDIS->rcItem);*/

	int nBkMode = pDC->SetBkMode(TRANSPARENT);

	if (lpDIS->itemState & ODS_SELECTED)
		crText = pDC->SetTextColor(::GetSysColor(COLOR_WINDOW));
	else if (lpDIS->itemState & ODS_DISABLED)
		crText = pDC->SetTextColor(::GetSysColor(COLOR_GRAYTEXT));
	else
		crText = pDC->SetTextColor(lpDIS->itemID == pMainFrame->Current ? RGB(0, 128, 128) : ::GetSysColor(COLOR_WINDOWTEXT));

	sText = m_PlaylistManager.GetTag(lpDIS->itemID);
	if (sText.GetLength() <= 9)
		sText = m_PlaylistManager.GetFilename(lpDIS->itemID);
	CRect rect;
	UINT nFormat;

	CString temp;
	temp.Format(_T("%d"), lpDIS->itemID + 1);

	long digits, q = m_PlaylistManager.GetSize(), chw = LOWORD(GetDialogBaseUnits());
	for (digits = 1; digits < 4; digits++)
	{
		if ((q = (int)(q / 10)) == 0)
			break;
	}

	rect = lpDIS->rcItem;
	rect.right = rect.left + digits * chw;

	nFormat = DT_RIGHT | DT_SINGLELINE | DT_VCENTER;

	pDC->DrawText(temp, -1, &rect, nFormat | DT_NOPREFIX);

	rect = lpDIS->rcItem;
	rect.left = rect.left + digits * chw + chw;
	rect.right = rect.right - (long)(chw * 4.5);

	nFormat = DT_LEFT | DT_SINGLELINE | DT_VCENTER;

	pDC->DrawText(sText, -1, &rect, nFormat | DT_NOPREFIX);

	pDC->SetTextColor(crText); 
	pDC->SetBkMode(nBkMode);
}

int CPLLB::GetFirstSelected(int from)
{
	int q = GetCount(), i;
	for (i = from; i < q; i++)
	{
		if (GetSel(i) > 0)
			return i;
	}
	return -1;
}

void CPLLB::MeasureItem(LPMEASUREITEMSTRUCT lpMIS)
{
	// Get current font metrics
	TEXTMETRIC metrics;
	HDC dc = ::GetDC(PlaylistDlg->m_hWnd);
	GetTextMetrics(dc, &metrics);
	::ReleaseDC(m_hWnd, dc);

	// Set the height
	lpMIS->itemHeight = metrics.tmHeight + metrics.tmExternalLeading;
}



void CPLLB::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	switch (nChar)
	{
	case VK_UP:
	case VK_DOWN:
	case VK_PRIOR:
	case VK_NEXT:
	case VK_HOME:
	case VK_END:
		{
			CListBox::OnKeyDown(nChar, nRepCnt, nFlags);
			if (ShiftDown)
			{
				int now = GetCaretIndex();
				if (prev == -1)
					prev = now;
				if (prev < now)
				{
					SetSel(-1, FALSE);
					SelItemRange(TRUE, prev, now);
				}
				else
				{
					SetSel(-1, FALSE);
					SelItemRange(TRUE, now, prev);
				}
			}
			else
			{
				SetSel(-1, FALSE);
				SetSel(GetCaretIndex(), TRUE);
			}
			PlaylistDlg->ShowFileName();
			break;
		}
	case VK_ADD:
		SetCaretIndex(0);
		SetSel(-1, TRUE);
		PlaylistDlg->ShowFileName();
		break;
	case VK_MULTIPLY:
		{
			int e = GetTopIndex();
			int w = GetCaretIndex();
			int q = GetCount();
			for (int i = 0; i < q; i++)
				SetSel(i, !GetSel(i));
			PlaylistDlg->ShowFileName();
			SetCaretIndex(w);
			SetTopIndex(e);
			Invalidate();
			break;
		}
	case VK_DELETE:
		PlaylistDlg->Remove();
		break;
	case VK_SHIFT:
		prev = GetCaretIndex();
		ShiftDown = true;
		break;
	case VK_CONTROL:
		CtrlDown = true;
		break;
	case 'A':
	case 'B':
	case 'C':
	case 'D':
	case 'E':
	case 'F':
	case 'G':
	case 'H':
	case 'I':
	case 'J':
	case 'K':
	case 'L':
	case 'M':
	case 'N':
	case 'O':
	case 'P':
	case 'Q':
	case 'R':
	case 'S':
	case 'T':
	case 'U':
	case 'V':
	case 'W':
	case 'X':
	case 'Y':
	case 'Z':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '0':
		PlaylistDlg->GetDlgItem(IDC_SEARCH)->SetFocus();
		PlaylistDlg->GetDlgItem(IDC_SEARCH)->PostMessage(WM_KEYDOWN, nChar, nRepCnt + 0x10000 * nFlags);
		return;
		break;
	}
}

void CPLLB::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags)
{
	if (nChar == VK_SHIFT)
	{
		ShiftDown = false;
		prev = -1;
	}
	if (nChar == VK_CONTROL)
		CtrlDown = false;
	//CListBox::OnKeyUp(nChar,nRepCnt,nFlags);
}

void CPLLB::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	BOOL q;
	int item;
	item = ItemFromPoint(point, q);
	if (!q)
	{
		PlaylistDlg->PlayTheFile(item);
		RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
	}
	CListBox::OnLButtonDblClk(nFlags, point);
}

void CPLLB::OnLButtonDown(UINT nFlags, CPoint point)
{
	BOOL q;
	posfrom = ItemFromPoint(point, q);
	if (q)
		posfrom = -1;
	if (!(nFlags & MK_CONTROL) && !(nFlags & MK_SHIFT))
		SetSel(-1, FALSE);
	int last = GetCaretIndex();
	if (nFlags & MK_SHIFT)
	{
		if (last != posfrom)
		{
			SetSel(-1, FALSE);
			if (posfrom < last)
				SelItemRange(TRUE, posfrom, last);
			else
				SelItemRange(TRUE, last, posfrom);
		}
	}
	else if (nFlags & MK_CONTROL)
	{
		SetCaretIndex(posfrom);
		SetSel(posfrom, !GetSel(posfrom));
	}
	else if (posfrom > -1)
	{
		SetCaretIndex(posfrom);
		SetSel(posfrom, TRUE);
	}
	PlaylistDlg->ShowFileName();
}

void CPLLB::OnMouseMove(UINT nFlags, CPoint point)
{
	if (nFlags & MK_LBUTTON)
	{
		CString buf;
		BOOL q;
		CPoint point2;
		CRect rect;
		GetClientRect(&rect);
		int nTargetIndex = ItemFromPoint(point, q);
		for (; ;)
		{
			if (!q)
			{
				if (posfrom >= 0 && nTargetIndex >= 0)
				{
					if (point.y - rect.top > 0 && point.y - rect.top < 10 && nTargetIndex > 0)
						nTargetIndex--;
					if (rect.bottom - point.y > 0 && rect.bottom - point.y < 10 && nTargetIndex < GetCount() - 1)
						nTargetIndex++;
					if (posfrom > nTargetIndex)
					{
						int offset = posfrom - nTargetIndex;
						if (!GetSel(0))
							for (UINT i = 1; i < (UINT)GetCount(); i++)
							{
								if (GetSel(i))
								{
									offset = (i - offset < 0 ? i : offset);
									CachedTag nTag = m_PlaylistManager.Get(i);
									m_PlaylistManager.RemoveAt(i);
									m_PlaylistManager.InsertAt(nTag, i - offset);
									SetSel(i, FALSE);
									SetSel(i - offset, TRUE);

									if (pMainFrame->Current == i)
										pMainFrame->Current = (i - offset);
									else if (pMainFrame->Current < i && pMainFrame->Current >= (i - offset))
										pMainFrame->Current++;
								}
							}
					}
					if (posfrom < nTargetIndex)
					{
						UINT offset = nTargetIndex - posfrom;
						if (!GetSel(GetCount() - 1))
							for (int i = (GetCount() - 2); i >= 0; i--)
							{
								if (GetSel(i))
								{
									offset = (i + offset < (UINT)GetCount() ? offset : GetCount() - 1 - i);
									CachedTag nTag = m_PlaylistManager.Get(i);
									m_PlaylistManager.RemoveAt(i);
									m_PlaylistManager.InsertAt(nTag, i + offset);
									SetSel(i, FALSE);
									SetSel(i + offset, TRUE);

									if (pMainFrame->Current == i)
										pMainFrame->Current = (i + offset);
									else if (pMainFrame->Current > (UINT)i && pMainFrame->Current <= (i + offset))
										pMainFrame->Current--;
								}
							}
					}
					Invalidate();
					posfrom = nTargetIndex;
				}
			}
			::GetCursorPos(&point2);
			ScreenToClient(&point2);
			nTargetIndex = ItemFromPoint(point, q);
			if (point2.y - rect.top > 0 && point2.y - rect.top < 3 || rect.bottom - point2.y > 0 && rect.bottom - point2.y < 3)
				RedrawWindow();
			else
				break;
		}
	}
	CListBox::OnMouseMove(nFlags, point);
}

void CPLLB::OnPaint()
{
	CPaintDC dc(this);

	if (!MemDCCreated)
	{
		memDC.CreateCompatibleDC(&dc);
		CRect rect;
		GetClientRect(&rect);
		memBitmap.CreateCompatibleBitmap(&dc, width = rect.Width(), height = rect.Height());
		memDC.SelectObject(&memBitmap);
		MemDCCreated = true;
	}

	memDC.FillSolidRect(0, 0, width, height, ::GetSysColor(COLOR_WINDOW));

	DefWindowProc(WM_PAINT, (WPARAM)memDC.m_hDC, (LPARAM)0);

	dc.BitBlt(0, 0, width, height, &memDC, 0, 0, SRCCOPY);
	PlaylistDlg->ShowTimes();
}

void CPLLB::OnRButtonDown(UINT nFlags, CPoint point)
{
	BOOL q;
	posfrom = ItemFromPoint(point, q);
	if (q)
		posfrom = -1;
	if (!(nFlags & MK_CONTROL) && !(nFlags & MK_SHIFT))
		SetSel(-1, FALSE);
	int last = GetCaretIndex();
	if (nFlags & MK_SHIFT)
	{
		if (last != posfrom)
		{
			SetSel(-1, FALSE);
			if (posfrom < last)
				SelItemRange(TRUE, posfrom, last);
			else
				SelItemRange(TRUE, last, posfrom);
		}
	}
	else if (posfrom > -1)
	{
		SetFocus();
		SetCaretIndex(posfrom);
		SetSel(posfrom, TRUE);
	}

	PlaylistDlg->ShowFileName();
}

void CPLLB::OnRButtonUp(UINT nFlags, CPoint point)
{
	CMenu* pPopup;
	menu->LoadMenu(IDR_PLPOPUP);
	pPopup = menu->GetSubMenu(0);
	m_Helper.translateMenu(pPopup, IDR_PLPOPUP);
	int i = PlaylistDlg->m_playlistat.GetFirstSelected();
	pPopup->EnableMenuItem(ID_FILE_REMOVE, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	pPopup->EnableMenuItem(ID_FILE_EDITTAG, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	pPopup->EnableMenuItem(ID_TRACKS_SELECTION_CREATENEWPLAYLIST, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);
	pPopup->EnableMenuItem(ID_TRACKS_SELECTION_ADDTOEXISTENTPLAYLIST, i == -1 ? MF_DISABLED | MF_GRAYED : MF_ENABLED);

	/*  	 if(m_PlaylistManager.GetSize()>0&&m_PlaylistManager.GetSize()<=50000)
		   {
			   CMenu *pTempMenu;
			   pTempMenu=pPopup->GetSubMenu(11);
			   pTempMenu->DeleteMenu(0,MF_BYPOSITION);
			   pMainFrame->BuildMenuLevel(pTempMenu->m_hMenu,pMainFrame->GetStartLevel(),0,"");
		   } else { pPopup->DeleteMenu(11,MF_BYPOSITION); pPopup->DeleteMenu(12,MF_BYPOSITION);}
	*/
	ClientToScreen(&point);
	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, PlaylistDlg);
	menu->DestroyMenu();
}

