// SettingsDialogInput.cpp : fichier d'implmentation
//

#include "stdafx.h"
#include "SysTrayPlay.h"
#include "SettingsDialog.h"
#include "SettingsDialogInput.h"
#include "MainHelper.h"
#include "MainWnd.h"
#include "volume.h"
#include "In/MultiDllInput.h"
#include "out/MultiDlloutput.h"
#include "WAEmuwnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CVolume* pVolume;

extern CMainWnd* pMainFrame;
extern Helper m_Helper;

extern CMultiDllInput* stdIn;
extern CMultiDllOutput* stdOut;
extern CWAEmuWnd* WAEmuWnd;

// Bote de dialogue CSettingsDialogInput


IMPLEMENT_MENUXP(CSettingsDialogInput, CPropertyPage);

IMPLEMENT_DYNAMIC(CSettingsDialogInput, CPropertyPage)

CSettingsDialogInput::CSettingsDialogInput() : CPropertyPage(IDD_SETTINGS_INPUT)
{
	m_gap = pMainFrame->GapBtwTracks;
	m_current = -1;
	updateOnly = false;
}

CSettingsDialogInput::~CSettingsDialogInput()
{
}

void CSettingsDialogInput::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	CMenuXP::SetXPLookNFeel(this, pMainFrame->XPLookNFeel);
	DDX_Text(pDX, IDC_GAP, m_gap);
	DDX_Control(pDX, IDC_ABOUT_INPUT, m_btnAbout);
	DDX_Control(pDX, IDC_CONFIG_INPUT, m_btnConfiguration);
	DDX_Control(pDX, IDC_LOAD_INPUT, m_btnLoad);
	DDX_Control(pDX, IDC_UNLOAD_INPUT, m_btnUnload);
	DDX_Control(pDX, IDC_LIST_INPUT, m_ListInput);
	DDX_Control(pDX, IDC_SPIN1, m_Spin);
}

BOOL CSettingsDialogInput::OnInitDialog()
{
	CPropertyPage::OnInitDialog();
	CMenuXP::SetXPLookNFeel(this, pMainFrame->XPLookNFeel);

	m_Helper.translateDlgItemText(this, IDC_LGAP);
	m_Helper.translateDlgItemText(this, IDC_LMILLISECONDS);
	m_Helper.translateDlgItemText(this, IDC_CONFIG_INPUT);
	m_Helper.translateDlgItemText(this, IDC_ABOUT_INPUT);
	m_Helper.translateDlgItemText(this, IDC_LOAD_INPUT);
	m_Helper.translateDlgItemText(this, IDC_UNLOAD_INPUT);

	((CSpinButtonCtrl *)GetDlgItem(IDC_SPIN))->SetRange(0, 10000);
	m_Spin.SetRange(0, stdIn->GetSize() - 1);
	OnInitList();
	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CSettingsDialogInput::OnInitList()
{
	for (int i = 0; i < stdIn->GetSize(); i++)
	{
		CDllInput* input = stdIn->getDllInput(i);
		CString desc = _T("");
		desc.Format(_T("%s (%s)"), input->GetDescription(), input->GetFileName());
		m_ListInput.AddString(desc);
		if (stdIn->GetIn_Module() == input->GetIn_Module())
		{
			m_ListInput.SetCurSel(i);
			m_Spin.SetPos(i);
			m_current = i;
		}
	}
}

void CSettingsDialogInput::OnOK()
{
	UpdateData(TRUE);

	pMainFrame->GapBtwTracks = m_gap;
#ifndef _UNICODE
	CString dllInKey = "";
	for (int i = 0; i < stdIn->GetSize(); i++)
	{
		CDllInput* input = stdIn->getDllInput(i);
		dllInKey.Format(_T("dllInName%d"), i);
		AfxGetApp()->WriteProfileString(_T("Preferences"), dllInKey, input->GetFilePath());
	}
	dllInKey.Format(_T("dllInName%d"), stdIn->GetSize());
	AfxGetApp()->WriteProfileString(_T("Preferences"), dllInKey, _T("NONE"));
#endif
	CPropertyPage::OnOK();
}
BEGIN_MESSAGE_MAP(CSettingsDialogInput, CPropertyPage)
  ON_BN_CLICKED(IDC_CONFIG_INPUT, OnBnClickedConfig)
  ON_BN_CLICKED(IDC_ABOUT_INPUT, OnBnClickedAbout)
  ON_BN_CLICKED(IDC_LOAD_INPUT, OnBnClickedLoad)
  ON_BN_CLICKED(IDC_UNLOAD_INPUT, OnBnClickedUnload)
  ON_EN_CHANGE(IDC_EDIT1,OnSpinChange)
  ON_LBN_SELCHANGE(IDC_LIST_INPUT,OnListChange)
  ON_MENUXP_MESSAGES()
END_MESSAGE_MAP()


// Gestionnaires de messages CSettingsDialogInput
void CSettingsDialogInput::OnBnClickedConfig()
{
#ifndef _UNICODE
	for (int i = stdIn->GetSize() - 1; i >= 0; i--)
	{
		if (m_ListInput.GetSel(i) != 0)
		{
			CDllInput* input = stdIn->getDllInput(i);
			input->Config(this->GetParent()->GetParent()->m_hWnd);
		}
	}
#else
	AfxMessageBox(m_Helper.MyLoadString(IDS_UNICODEERROR), MB_OK | MB_ICONERROR);
#endif
}

void CSettingsDialogInput::OnBnClickedAbout()
{
#ifndef _UNICODE
	for (int i = stdIn->GetSize() - 1; i >= 0; i--)
	{
		if (m_ListInput.GetSel(i) != 0)
		{
			CDllInput* input = stdIn->getDllInput(i);
			input->About(this->GetParent()->GetParent()->m_hWnd);
		}
	}
#else
	AfxMessageBox(m_Helper.MyLoadString(IDS_UNICODEERROR), MB_OK | MB_ICONERROR);
#endif
}

void CSettingsDialogInput::OnBnClickedLoad()
{
#ifndef _UNICODE
	CString filter = "";
	CFileDialog dialog(TRUE, _T("dll"), _T(""), OFN_FILEMUSTEXIST, m_Helper.MyLoadString(IDS_OPENINPUT));
	TCHAR buffer[_MAX_PATH*500 + 1];
	buffer[0] = '\0';
	dialog.m_ofn.lpstrFile = buffer;
	dialog.m_ofn.nMaxFile = _MAX_PATH * 500;
	if (dialog.DoModal() == IDOK)
	{
		int oldSize = stdIn->GetSize();
		stdIn->Load(dialog.GetPathName(), WAEmuWnd->m_hWnd);
		if (oldSize != stdIn->GetSize())
		{
			CDllInput* input = stdIn->getDllInput(oldSize);
			stdIn->SetOutMod(stdOut->GetOut_Module());
			CString desc = _T("");
			desc.Format(_T("%s (%s)"), input->GetDescription(), input->GetFileName());
			m_ListInput.AddString(desc);
			m_Spin.SetRange(0, stdIn->GetSize() - 1);
			m_current = -1;
			m_ListInput.SetCurSel(m_current);
		}
	}
#else
	AfxMessageBox(m_Helper.MyLoadString(IDS_UNICODEERROR), MB_OK | MB_ICONERROR);
#endif
}

void CSettingsDialogInput::OnBnClickedUnload()
{
#ifndef _UNICODE
	if (-1 < m_ListInput.GetCurSel() && m_ListInput.GetCurSel() < stdIn->GetSize())
	{
		CDllInput* input = stdIn->getDllInput(m_ListInput.GetCurSel());
		if (input->GetFilePath() != "STANDARD")
		{
			if (input->GetIn_Module() == stdIn->GetIn_Module() && input->GetPlayerMode() != PLAYER_MODE_CLOSE)
				::SendMessage(WAEmuWnd->m_hWnd, WM_COMMAND, WINAMP_BUTTON4, 0);
			stdIn->UnLoad(m_ListInput.GetCurSel());
			m_ListInput.DeleteString(m_ListInput.GetCurSel());
			m_Spin.SetRange(0, stdIn->GetSize() - 1);
			m_current = -1;
			m_ListInput.SetCurSel(m_current);
		}
	}
#else
	AfxMessageBox(m_Helper.MyLoadString(IDS_UNICODEERROR), MB_OK | MB_ICONERROR);
#endif
}

void CSettingsDialogInput::OnSpinChange()
{
#ifndef _UNICODE
	if (!updateOnly && m_current != -1)
	{
		if (m_current >= 0 && m_current <= stdIn->GetSize() - 1)
		{
			if (m_current > m_Spin.GetPos())
			{
				stdIn->UpDllInput(m_current);
				CDllInput* input = stdIn->getDllInput(m_current);
				CString desc = "";
				desc.Format(_T("%s (%s)"), input->GetDescription(), input->GetFileName());
				m_ListInput.DeleteString(m_Spin.GetPos());
				m_ListInput.InsertString(m_current, desc);
			}
			else
			{
				stdIn->UpDllInput(m_Spin.GetPos());
				CDllInput* input = stdIn->getDllInput(m_current);
				CString desc = "";
				desc.Format(_T("%s (%s)"), input->GetDescription(), input->GetFileName());
				m_ListInput.DeleteString(m_Spin.GetPos());
				m_ListInput.InsertString(m_current, desc);
			}
			m_current = m_Spin.GetPos();
			m_ListInput.SetCurSel(m_current);
		}
	}
	updateOnly = false;
#endif
}
void CSettingsDialogInput::OnListChange()
{
#ifndef _UNICODE
	m_current = m_ListInput.GetCurSel();
	updateOnly = true;
	m_Spin.SetPos(m_current);
#else
	AfxMessageBox(m_Helper.MyLoadString(IDS_UNICODEERROR), MB_OK | MB_ICONERROR);
#endif
}
